#ifndef DISKCACHE__QUERY_RESULTS_HH
#define DISKCACHE__QUERY_RESULTS_HH

namespace diskCache
{
  class QueryResults
  {
  public:
    enum type_type {
      //-----------------------------------------------------------------
      /// \brief Type is for internal use for a specific filename handler.
      ///
      /// This type of query results is intended for internal use only
      /// and is only understood by the filename handler in which it
      /// was defined.
      ///
      /// Because the type cannot be determined by the limited
      /// enumerated types given here,
      /// accessing of the internal information will require this
      /// object to be dynamically cast.
      //-----------------------------------------------------------------
      TYPE_INTERNAL,
      TYPE_DUMP,
      TYPE_FILENAMES,
      TYPE_INTERVALS,
      TYPE_MOUNT_POINT_STATS,
      TYPE_TIME_RANGE
    };

    enum format_type{
      /// Format results to be easily parsed by TCL.
      FORMAT_TCL,
      /// Throw exception if anything is added as it means data
      /// is in conflict.
      FORMAT_CONFLICT
    };

    QueryResults( format_type Format, type_type Type );

    virtual ~QueryResults( );

    format_type Format( ) const;

    type_type Type( ) const;

    void tcl_list_begin( std::ostringstream& Stream ) const;

    void tcl_list_begin( std::string& String ) const;

    void tcl_list_end( std::ostringstream& Stream ) const;

    void tcl_list_end( std::string& String ) const;

  protected:
    format_type format( ) const;

    const std::string& formatString( ) const;

    void tcl_make_list( std::string& Data ) const;

  private:
    const format_type		m_format;
    const type_type		m_type;
  };

  inline QueryResults::
  ~QueryResults( )
  {
  }

  inline QueryResults::format_type QueryResults::
  Format( ) const
  {
    return m_format;
  }

  inline QueryResults::type_type QueryResults::
  Type( ) const
  {
    return m_type;
  }

  inline QueryResults::format_type QueryResults::
  format( ) const
  {
    return m_format;
  }
  
} // namespace - diskCache

#endif /* DISKCACHE__QUERY_RESULTS_HH */
