#ifndef FrameCPP__Common__FrHeader_HH
#define	FrameCPP__Common__FrHeader_HH

#include <algorithm>
#include <string>

#include "general/types.hh"

#include "framecpp/Common/FrameSpec.hh"

#include "framecpp/Common/IOStream.hh"

namespace FrameCPP
{
  namespace Common
  {
    class FrHeader
    {
    public:
      typedef INT_2U version_type;
      typedef INT_2U library_revision_type;
      typedef CHAR* originator_type;

      FrHeader( IStream& Stream );

      virtual ~FrHeader( );
      virtual cmn_streamsize_type Bytes( ) const;

      virtual bool ByteSwapping( ) const;

      virtual void Filter( StreamFilter& SFilter ) const;

      library_revision_type GetLibraryRevision( ) const;
      version_type GetVersion( ) const;
      const originator_type GetOriginator( ) const;

      /// \brief Establish the version of the Library used
      void SetLibraryVersion( INT_2U Version );

      /// \brief Establish the originator of the frame.
      void SetOriginator( const std::string& Originator );

      /// \brief Establish the version of the frame specification.
      void SetVersion( INT_2U Version );

      void Write( OStream& Stream ) const;
      
      const FrHeader& operator=( const FrHeader& Source );

    protected:
      FrHeader( );
      FrHeader( const FrHeader& Source );

      //-----------------------------------------------------------------
      /// \brief Entry point to do extra processing
      //-----------------------------------------------------------------
      virtual void complete( );

    private:
      union data_type {
	struct raw_type
	{
	  CHAR		originator[ 5 ];
	  CHAR_U	version;
	  CHAR_U	version_minor;
	} raw;
      };

      data_type	m_data;
    }; // class - FrHeader

    inline FrHeader::
    FrHeader( const FrHeader& Source )
    {
      const char* s = reinterpret_cast< const char* >( &(Source.m_data.raw) );
      char* d = reinterpret_cast< char* >( &(m_data.raw) );

      std::copy( s, s + sizeof( m_data.raw ), d );
    }

    inline FrHeader::
    FrHeader( )
    {
    }

    inline FrHeader::
    ~FrHeader( )
    {
    }

    inline cmn_streamsize_type FrHeader::
    Bytes( ) const
    {
      return
	sizeof( m_data.raw.originator )
	+ sizeof( m_data.raw.version )
	+ sizeof( m_data.raw.version_minor )
	;
    }

    inline void FrHeader::
    Filter( StreamFilter& SFilter ) const
    {
      SFilter.pubfilter( reinterpret_cast< const char* >( &m_data.raw ),
			 sizeof( m_data.raw ) );
    }

    inline const FrHeader::originator_type FrHeader::
    GetOriginator( ) const
    {
      return (const FrHeader::originator_type)( &m_data.raw.originator[ 0 ] );
    }
    inline FrHeader::library_revision_type FrHeader::
    GetLibraryRevision( ) const
    {
      return m_data.raw.version_minor;
    }

    inline FrHeader::version_type FrHeader::
    GetVersion( ) const
    {
      return m_data.raw.version;
    }

    inline void FrHeader::
    SetLibraryVersion( INT_2U Version )
    {
      m_data.raw.version_minor = Version;
    }

    inline void FrHeader::
    SetVersion( INT_2U Version )
    {
      m_data.raw.version = Version;
    }

    inline const FrHeader& FrHeader::
    operator=( const FrHeader& Source )
    {
      const char* sptr = reinterpret_cast< const char* >( &Source.m_data.raw );
      std::copy( sptr, sptr + sizeof( m_data.raw ),
		 reinterpret_cast< char* >( &m_data.raw ) );
      complete( );
      return *this;
    }

  } // namespace - Common
} // namespace - FrameCPP

#endif /* FrameCPP__Common__FrHeader_HH */
