#ifndef FrameCPP__COMMON__FR_TOC_HH
#define FrameCPP__COMMON__FR_TOC_HH

#include "framecpp/Common/FrameSpec.hh"
#include "framecpp/Common/STRING.hh"
#include "framecpp/Common/FrStatData.hh"

namespace FrameCPP
{
  namespace Common
  {
    class Description;

    class FrTOC
      : public FrameSpec::Object
    {
    public:
      enum query_info_type {
	TOC_CHANNEL_NAMES,
	TOC_DETECTOR,
	TOC_FR_STRUCTS
      };

      class FunctionBase
      {
      public:
	virtual ~FunctionBase( );
      };
      
      //-----------------------------------------------------------------
      /// \brief Function class used by ForEach method
      ///
      /// This function class is used to describe a function that
      /// takes as its parameters a single character argument.
      //-----------------------------------------------------------------
      class FunctionC
	: public FunctionBase
      {
      public:	
	//---------------------------------------------------------------
	/// \brief Destructor
	//---------------------------------------------------------------
	virtual ~FunctionC( );

	//---------------------------------------------------------------
	/// \brief Prototype for function.
	//---------------------------------------------------------------
	virtual void operator()( char C ) = 0;
      };

      class FunctionString
	: public FunctionBase
      {
      public:
	virtual ~FunctionString( );

	virtual void operator()( const std::string& StrObj ) = 0;
      };

      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      class FunctionSI
	: public FunctionBase
      {
      public:	
	//---------------------------------------------------------------
	/// \brief Destructor
	//---------------------------------------------------------------
	virtual ~FunctionSI( );

	//---------------------------------------------------------------
	/// \brief Prototype for function.
	//---------------------------------------------------------------
	virtual void operator()( const std::string& S, INT_4U I ) = 0;
      };

      static const Common::FrameSpec::Info::frame_object_types s_object_id
      = Common::FrameSpec::Info::FSI_FR_TOC;

      FrTOC( const Description* Desc );

      typedef std::vector< STRING< INT_2U > > cmn_name_container_type;
      typedef std::vector< REAL_8 > cmn_dt_container_type;
      typedef std::vector< INT_4U > cmn_GTimeS_container_type;
      typedef std::vector< INT_4U > cmn_GTimeN_container_type;

      virtual ~FrTOC( );

      
      virtual void ForEach( query_info_type Info, FunctionBase& Action ) const;

      virtual void IndexObject( const FrameSpec::Object* Obj,
				std::streampos Position ) = 0;

      //-----------------------------------------------------------------
      // 
      //-----------------------------------------------------------------

      virtual void FrStatDataQuery( const std::string& NamePattern,
				    const General::GPSTime& StartTime,
				    const General::GPSTime& EndTime,
				    const INT_4U Version,
				    FrStatData::Query& Result ) const = 0;

      //-----------------------------------------------------------------
      // Interface routines
      //-----------------------------------------------------------------
      virtual INT_4U nFrame( ) const = 0;

      virtual const cmn_dt_container_type& dt( ) const = 0;
      virtual const cmn_GTimeS_container_type& GTimeS( ) const = 0;
      virtual const cmn_GTimeN_container_type& GTimeN( ) const = 0;

      virtual INT_8U positionH( INT_4U FrameIndex ) const = 0;
      virtual const cmn_name_container_type& nameADC( ) const = 0;
      virtual INT_8U positionADC( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;
      virtual INT_8U positionADC( INT_4U FrameIndex,
				  INT_4U Channel ) const = 0;
      virtual INT_8U positionEvent( INT_4U FrameIndex,
				    const std::string& Channel ) const = 0;
      virtual const cmn_name_container_type& nameProc( ) const = 0;
      virtual INT_8U positionProc( INT_4U FrameIndex,
				   const std::string& Channel ) const = 0;
      virtual INT_8U positionProc( INT_4U FrameIndex,
				   INT_4U Channel ) const = 0;
      virtual const cmn_name_container_type& nameSim( ) const = 0;
      virtual INT_8U positionSim( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;
      virtual const cmn_name_container_type& nameSer( ) const = 0;
      virtual INT_8U positionSer( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;
      virtual INT_8U positionSimEvent( INT_4U FrameIndex,
				       const std::string& Channel ) const = 0;

      virtual INT_4U nSH( ) const = 0;
      virtual INT_2U SHid( INT_4U Offset ) const = 0;
      virtual const std::string& SHname( INT_4U Offset ) const = 0;

    protected:

    private:
    };
  } // namespace - Common
} // namespace - FrameCPP

#endif /* FrameCPP__COMMON__FR_TOC_HH */
