#include "framecpp/config.h"

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"
#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version4/FrAdcData.hh"
#include "framecpp/Version4/FrSE.hh"
#include "framecpp/Version4/FrSH.hh"

#include "framecpp/Version4/PTR_STRUCT.hh"

using namespace FrameCPP::Version_4;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;


using std::ceil;
using std::floor;

//-----------------------------------------------------------------------
// Local functions and variables
//-----------------------------------------------------------------------
static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_ADC_DATA;

static INT_2U MAX_REF = 2;

//-----------------------------------------------------------------------
// FrAdcData::data_definition_type
//-----------------------------------------------------------------------
FrAdcData::data_definition_type::
data_definition_type( )
{
}

FrAdcData::data_definition_type::
data_definition_type( const data_definition_type& Source )
  : data( Source.data )
{
  copy_core( Source );
}

bool FrAdcData::data_definition_type::
operator==( const data_definition_type& RHS ) const
{
#if LM_INFO
#define LM_INFO_CHECK(a) \
  if ( ! ( a == RHS.a ) ) \
  { \
    std::cerr << "DEBUG: Comparison of " << #a << " failed" << std::endl; \
  }

  LM_INFO_CHECK( name );
  LM_INFO_CHECK( comment);
  LM_INFO_CHECK( channelGroup );
  LM_INFO_CHECK( channelNumber );
  LM_INFO_CHECK( nBits );
  LM_INFO_CHECK( bias );
  LM_INFO_CHECK( slope );
  LM_INFO_CHECK( units );
  LM_INFO_CHECK( fShift );
  LM_INFO_CHECK( timeOffset );
  LM_INFO_CHECK( dataValid );
  LM_INFO_CHECK( phase );
  LM_INFO_CHECK( data );
  LM_INFO_CHECK( aux );
#undef LM_INFO_CHECK
#endif /* LM_INFO */

  const bool	retval =
    ( ( this == &RHS ) ||	// Short circuit for comparison to self
      ( ( name == RHS.name ) &&
	( comment == RHS.comment ) &&
	( channelGroup == RHS.channelGroup ) &&
	( channelNumber == RHS.channelNumber ) &&
	( nBits == RHS.nBits ) &&
	( sampleRate == RHS.sampleRate ) &&
	( bias == RHS.bias ) &&
	( slope == RHS.slope ) &&
	( units == RHS.units ) &&
	( fShift == RHS.fShift ) &&
	( timeOffsetS == RHS.timeOffsetS ) &&
	( timeOffsetN == RHS.timeOffsetN ) &&
	( dataValid == RHS.dataValid ) &&
	( data == RHS.data ) &&
	( aux == RHS.aux ) ) );

  return retval;

}

bool FrAdcData::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

void FrAdcData::data_definition_type::
copy_core( const data_definition_type& Source )
{
  name = Source.name;
  comment = Source.comment;
  channelGroup = Source.channelGroup;
  channelNumber = Source.channelNumber;
  nBits = Source.nBits;
  sampleRate = Source.sampleRate;
  bias = Source.bias;
  slope = Source.slope;
  units = Source.units;
  fShift = Source.fShift;
  timeOffsetS = Source.timeOffsetS;
  timeOffsetN = Source.timeOffsetN;
  dataValid = Source.dataValid;
  aux = Source.aux;
}

//-----------------------------------------------------------------------
//
//-----------------------------------------------------------------------

FrAdcData::
FrAdcData( )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
}

FrAdcData::
FrAdcData( const FrAdcData& Source )
  : FrameSpec::Object( s_object_id, StructDescription( ) ),
    m_data( Source.m_data )
{
}

FrAdcData::
FrAdcData( const std::string& name, INT_4U group,
	   INT_4U channel, INT_4U nBits,
	   REAL_8 sampleRate,
	   REAL_4 bias,
	   REAL_4 slope,
	   const std::string& units,
	   REAL_8 fShift,
	   INT_4S timeOffsetS,
	   INT_4U timeOffsetN,
	   INT_2U dataValid )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = name;
  m_data.channelGroup = group;
  m_data.channelNumber = channel;
  m_data.nBits = nBits;
  m_data.sampleRate = sampleRate;
  m_data.bias = bias;
  m_data.slope = slope;
  m_data.units = units;
  m_data.fShift = fShift;
  m_data.timeOffsetS = timeOffsetS;
  m_data.timeOffsetN = timeOffsetN;
  m_data.dataValid = dataValid;
}

FrAdcData::
FrAdcData( Previous::FrAdcData& Source, IStream* Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = Source.GetName( );
  m_data.comment = Source.GetComment( );
  m_data.channelGroup = Source.GetCrate( );
  m_data.channelNumber = Source.GetChannel( );
  m_data.nBits = Source.GetNBits( );
  m_data.bias = Source.GetBias( );
  m_data.slope = Source.GetSlope( );
  m_data.units = Source.GetUnits( );
  m_data.sampleRate = Source.GetSampleRate( );
  m_data.timeOffsetS = Source.GetTimeOffsetS( );
  m_data.timeOffsetN = Source.GetTimeOffsetN( );
  m_data.fShift = Source.GetFShift( );
  m_data.dataValid = Source.GetOverRange( );

  if ( Stream )
  {
    //-------------------------------------------------------------------
    // Modify references
    //-------------------------------------------------------------------
      Stream->ReplaceRef( RefData( ), &(Source.RefData( )), MAX_REF );
      Stream->ReplaceRef( RefAux( ), &(Source.RefAux( )), MAX_REF );
  }
}

FrAdcData::
FrAdcData( IStream& Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
    Stream >> m_data.name
	   >> m_data.comment
	   >> m_data.channelGroup
	   >> m_data.channelNumber
	   >> m_data.nBits
	   >> m_data.bias
	   >> m_data.slope
	   >> m_data.units
	   >> m_data.sampleRate
	   >> m_data.timeOffsetS
	   >> m_data.timeOffsetN
	   >> m_data.fShift
	   >> m_data.dataValid
	   >> m_data.data
	   >> m_data.aux
      ;
  Stream.Next( this );
}

FrAdcData::
~FrAdcData( )
{
}

const std::string& FrAdcData::
GetName() const
{
   return m_data.name;
}


FrameCPP::cmn_streamsize_type FrAdcData::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.name.Bytes( ) +
    m_data.comment.Bytes( ) +
    sizeof( m_data.channelGroup ) +
    sizeof( m_data.channelNumber ) +
    sizeof( m_data.nBits ) +
    sizeof( m_data.bias ) +
    sizeof( m_data.slope ) +
    m_data.units.Bytes( ) +
    sizeof( m_data.sampleRate ) +
    sizeof( m_data.timeOffsetS ) +
    sizeof( m_data.timeOffsetN ) +
    sizeof( m_data.fShift ) +
    sizeof( m_data.dataValid ) +
    Stream.PtrStructBytes( ) +	// aux
    Stream.PtrStructBytes( ) +	// data
    Stream.PtrStructBytes( )	// next
    ;
}

FrAdcData* FrAdcData::
Create( IStream& Stream ) const
{
  return new FrAdcData( Stream );
}

FrAdcData& FrAdcData::
Merge( const FrAdcData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}

const char* const FrAdcData::
ObjectStructName( ) const
{
  return StructName( );
}


const Description* FrAdcData::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrAdcData::StructName( ), s_object_id,
	       "AdcData  Data Structure" ) );

    ret( FrSE( "name", "STRING",
	       "" ) );
    ret( FrSE( "comment", "STRING",
	       "" ) );
    ret( FrSE( "channelGroup", "INT_4U",
	       "" ) );
    ret( FrSE( "channelNumber", "INT_4U",
	       "" ) );
    ret( FrSE( "nBits", "INT_4U",
	       "" ) );
    ret( FrSE( "bias", "REAL_4",
	       "" ) );
    ret( FrSE( "slope", "REAL_4",
	       "" ) );
    ret( FrSE( "units", "STRING",
	       "" ) );
    ret( FrSE( "sampleRate", "REAL_8",
	       "" ) );
    ret( FrSE( "timeOffsetS", "INT_4S",
	       "" ) );
    ret( FrSE( "timeOffsetN", "INT_4U",
	       "" ) );
    ret( FrSE( "fShift", "REAL_8",
	       "" ) );
    ret( FrSE( "dataValid", "INT_2U",
	       "" ) );

    ret( FrSE( "data", PTR_STRUCT::Desc( FrVect::StructName( ) ),
	       ""  ) );
    ret( FrSE( "aux", PTR_STRUCT::Desc( FrVect::StructName( ) ),
	       ""  ) );

    ret( FrSE( "next",	PTR_STRUCT::Desc( FrAdcData::StructName( ) ),
	       ""  ) );
  }

  return &ret;
}

void FrAdcData::
Write( OStream& Stream ) const
{
  try
  {
    Stream << m_data.name
	   << m_data.comment
	   << m_data.channelGroup
	   << m_data.channelNumber
	   << m_data.nBits
	   << m_data.bias
	   << m_data.slope
	   << m_data.units
	   << m_data.sampleRate
	   << m_data.timeOffsetS
	   << m_data.timeOffsetN
	   << m_data.fShift
	   << m_data.dataValid
	   << m_data.data
	   << m_data.aux
      ;
    WriteNext( Stream );
  }
  catch ( const std::exception& Exception )
  {
    std::cerr << "DEBUG: Error Writing FrAdcData: " << m_data.name
	      << " exception: " << Exception.what( )
	      << std::endl;
    throw; // rethrow
  }
}

void FrAdcData::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  while ( InfoClass != TOCInfo::IC_EOQ )
  {
    int data_type = va_arg( vl, int );
    switch( data_type )
    {
    case TOCInfo::DT_STRING_2:
      {
	STRING* data = va_arg( vl, STRING* );
	switch( InfoClass )
	{
	case TOCInfo::IC_NAME:
	  *data = GetName( );
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    case TOCInfo::DT_INT_2U:
      {
	INT_2U* data = va_arg( vl, INT_2U* );
	switch( InfoClass )
	{
	case TOCInfo::IC_DATA_VALID:
	  *data = GetDataValid( );
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    case TOCInfo::DT_INT_4U:
      {
	INT_4U* data = va_arg( vl, INT_4U* );
	switch( InfoClass )
	{
	case TOCInfo::IC_CHANNEL_ID:
	  *data = GetChannelNumber( );
	  break;
	case TOCInfo::IC_GROUP_ID:
	  *data = GetChannelGroup( );
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    default:
      // Stop processing
      goto cleanup;
    }
    InfoClass = va_arg( vl, int );
  }
 cleanup:
  va_end( vl );
}

std::auto_ptr< FrameSpec::Object > FrAdcData::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrAdcData* >( this ) );
  }
  try
  {
    //-------------------------------------------------------------------
    // Copy non-reference information
    //-------------------------------------------------------------------

    // Do actual down conversion
    std::auto_ptr< Previous::FrAdcData >
      retval( new Previous::FrAdcData( GetName( ),
				       GetChannelGroup( ),
				       GetChannelNumber( ),
				       GetNBits( ),
				       GetSampleRate( ),
				       GetBias( ),
				       GetSlope( ),
				       GetUnits( ),
				       GetFShift( ),
				       GetTimeOffsetS( ), GetTimeOffsetN( ),
				       GetDataValid( )
				       ) )
      ;
    reinterpret_cast< Previous::FrAdcData* >(  retval.get( ) )->AppendComment( GetComment( ) );
    if ( Stream )
    {
      //-----------------------------------------------------------------
      // Modify references
      //-----------------------------------------------------------------
      Stream->ReplaceRef( retval->RefData( ), &(RefData( )), MAX_REF );
      Stream->ReplaceRef( retval->RefAux( ), &(RefAux( )), MAX_REF );
    }
    //-------------------------------------------------------------------
    // Return demoted object
    //-------------------------------------------------------------------
    return std::auto_ptr< FrameSpec::Object >( retval.release( ) );
  }
  catch( ... )
  {
  }
  throw
    Unimplemented( "Object* FrAdcData::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrAdcData::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}
